#include "Biped.h"
#include <iostream>
using std::cerr;
using std::endl;


Biped::~Biped()
{}



SoGroup* Biped::getRoot()
{
    return root;
}

void Biped::reset()
{
	bodyCenter = SbVec3f(0,hbody/2 + lthigh +ltibia,0);
	update( 0,0 );
}

Biped::Biped(
    float hc, float lc, float pc,
    float rt,
    float Lc, float rc,
    float lm, float rm,
    float lb, float rb,
    float lab,
    float p,
    float a
)
	: hbody(hc)
        , lbody(lc)
        , pbody(pc)
        , rhead(rt)
        , lthigh(Lc)
        , rthigh(rc)
        , ltibia(lm)
        , rtibia(rm)
        , lupper_arm(lb)
        , rupper_arm(rb)
        , llower_arm(lab)
        , period(p)
        , amplitude(a)
{
	bodyCenter = SbVec3f(0,hbody/2 + lthigh +ltibia,0); // position of the center of the body 

    root = new SoSeparator;

    position = new SoTransform;
    root->addChild(position);
	position->translation = bodyCenter;
    position->rotation.setValue( SbVec3f(0,1,0), M_PI/2 );

    SoSeparator* body = new SoSeparator;
    root->addChild( body );

    SoMaterial *bronze = new SoMaterial;
    body->addChild( bronze );
    bronze->ambientColor.setValue(.33, .22, .27);
    bronze->diffuseColor.setValue(.78, .57, .11);
    bronze->specularColor.setValue(.99, .94, .81);
    bronze->shininess = .28;

    SoSeparator* poitrine = new SoSeparator;
    body->addChild(poitrine);

    SoTransform* sein_left_transf = new SoTransform;
    poitrine->addChild( sein_left_transf );
    float xsein = lbody/6;
    sein_left_transf->translation.setValue( -xsein, hbody/4, pbody/2 );

    SoSphere* sein_sph = new SoSphere;
    poitrine->addChild( sein_sph );
    sein_sph->radius.setValue( lbody/6 );

    SoTransform* sein_right_transf = new SoTransform;
    poitrine->addChild( sein_right_transf );
    sein_right_transf->translation.setValue( 2*xsein,0,0 );

    poitrine->addChild( sein_sph );

    SoTransform* body_transf = new SoTransform;
    body->addChild( body_transf );
    body_transf->rotation.setValue( SbVec3f(0,0,1),3.1416 );
    body_transf->scaleFactor.setValue( 1,1, 1.5*pbody/lbody );

    SoCone* body_cone = new SoCone;
    body->addChild( body_cone );
    body_cone->bottomRadius = lbody/2;
    body_cone->height = hbody;

    SoTransform* jupe_transf = new SoTransform;
    body->addChild( jupe_transf );
    jupe_transf->translation.setValue( 0,hbody/4,0 );
    jupe_transf->rotation.setValue( SbVec3f(0,0,1),3.1416 );

    SoCone* jupe_cone = new SoCone;
    body->addChild(jupe_cone);
    jupe_cone->bottomRadius = lbody*0.6;
    jupe_cone->height = hbody*0.7;
    jupe_cone->parts = SoCone::SIDES;

    SoSeparator* head = new SoSeparator;
    root->addChild( head );

    SoMaterial *silver = new SoMaterial;
    head->addChild( silver );
    silver->ambientColor.setValue(.2, .2, .2);
    silver->diffuseColor.setValue(.6, .6, .6);
    silver->specularColor.setValue(.5, .5, .5);
    silver->shininess = .5;

    SoTransform* head_trans = new SoTransform;
    head->addChild( head_trans );
    head_trans->translation.setValue( 0, hbody/2 + rhead, 0 );

    SoSphere* head_sphere = new SoSphere;
    head->addChild( head_sphere );
    head_sphere->radius.setValue( rhead );


    SoSeparator* jambe_righte = new SoSeparator;
    root->addChild( jambe_righte );

    right_hip = new SoTransform;
    jambe_righte->addChild( right_hip );
    float shift_hanche = rthigh * 0.5;
    right_hip->translation.setValue( -shift_hanche, -hbody/2, 0 );
    right_hip->rotation.setValue( SbVec3f(1,0,0), -0.2 );

    SoSeparator* thigh = new SoSeparator;
    jambe_righte->addChild( thigh );

    SoTransform* thigh_pos = new SoTransform;
    thigh->addChild( thigh_pos );
    thigh_pos->translation.setValue( 0, -lthigh/2, 0 );
    thigh_pos->rotation.setValue( SbVec3f(0,0,1), 3.1416 );

    SoCone* thigh_cone = new SoCone;
    thigh->addChild( thigh_cone );
    thigh_cone->bottomRadius = rthigh;
    thigh_cone->height = lthigh;

    SoCylinder* thigh_cyl = new SoCylinder;
    thigh->addChild( thigh_cyl );
    thigh_cyl->radius = rthigh/2;
    thigh_cyl->height = lthigh;

    right_knee = new SoTransform;
    jambe_righte->addChild( right_knee );
    right_knee->translation.setValue( 0, -lthigh, 0 );
    right_knee->rotation.setValue( SbVec3f(1,0,0), 0.2 );

    SoSeparator* tibia = new SoSeparator;
    jambe_righte->addChild( tibia );

    SoTransform* tibia_pos = new SoTransform;
    tibia->addChild( tibia_pos );
    tibia_pos->translation.setValue( 0, -ltibia/2, 0 );
    tibia_pos->rotation.setValue( SbVec3f(0,0,1), 3.1416 );

    SoCone* tibia_cone = new SoCone;
    tibia->addChild( tibia_cone );
    tibia_cone->bottomRadius = rtibia;
    tibia_cone->height = ltibia;

    SoSeparator* jambe_left = new SoSeparator;
    root->addChild( jambe_left );

    left_hip = new SoTransform;
    jambe_left->addChild( left_hip );
    left_hip->translation.setValue( shift_hanche, -hbody/2, 0 );

    jambe_left->addChild( thigh );


    left_knee = new SoTransform;
    jambe_left->addChild( left_knee );
    left_knee->translation.setValue( 0, -lthigh, 0 );

    jambe_left->addChild( tibia );


    SoSeparator* upper_arm_right = new SoSeparator;
    root->addChild( upper_arm_right );

    right_shoulder = new SoTransform;
    upper_arm_right->addChild( right_shoulder );
    float shift_shoulder = lbody/2 - rupper_arm;
    right_shoulder->translation.setValue( -shift_shoulder, hbody/2, 0 );
    // right_shoulder->rotation.setValue( SbVec3f(1,0,0), -0.2 );

    SoSeparator* upper_arm = new SoSeparator;
    upper_arm_right->addChild( upper_arm );

    SoTransform* upper_arm_pos = new SoTransform;
    upper_arm->addChild( upper_arm_pos );
    upper_arm_pos->translation.setValue( 0, -lupper_arm/2, 0 );
    upper_arm_pos->rotation.setValue( SbVec3f(0,0,1), 3.1416 );

    SoCone* upper_arm_cone = new SoCone;
    upper_arm->addChild( upper_arm_cone );
    upper_arm_cone->bottomRadius = rupper_arm;
    upper_arm_cone->height = lupper_arm;

    SoCylinder* upper_arm_cyl = new SoCylinder;
    upper_arm->addChild( upper_arm_cyl );
    float rcoude = rupper_arm/1.5;
    upper_arm_cyl->radius = rcoude;
    upper_arm_cyl->height = lupper_arm;

    right_elbow = new SoTransform;
    upper_arm_right->addChild( right_elbow );
    right_elbow->translation.setValue( 0, -lupper_arm, 0 );
    right_elbow->rotation.setValue( SbVec3f(1,0,0), -0.2 );

    SoSeparator* lower_arm = new SoSeparator;
    upper_arm_right->addChild( lower_arm );

    SoTransform* lower_arm_pos = new SoTransform;
    lower_arm->addChild( lower_arm_pos );
    lower_arm_pos->translation.setValue( 0, -llower_arm/2, 0 );
    lower_arm_pos->rotation.setValue( SbVec3f(0,0,1), 3.1416 );

    SoCone* lower_arm_cone = new SoCone;
    lower_arm->addChild( lower_arm_cone );
    lower_arm_cone->bottomRadius = rcoude;
    lower_arm_cone->height = llower_arm;


    SoSeparator* upper_arm_left = new SoSeparator;
    root->addChild( upper_arm_left );

    left_shoulder = new SoTransform;
    upper_arm_left->addChild( left_shoulder );
    left_shoulder->translation.setValue( shift_shoulder, hbody/2, 0 );
    left_shoulder->rotation.setValue( SbVec3f(1,0,0), -0.2 );

    upper_arm_left->addChild( upper_arm );

    left_elbow = new SoTransform;
    upper_arm_left->addChild( left_elbow );
    left_elbow->translation.setValue( 0, -lupper_arm, 0 );
    left_elbow->rotation.setValue( SbVec3f(1,0,0), -0.2 );

    upper_arm_left->addChild( lower_arm );
	
	reset();

}

void Biped::interpolate( const float* initialValues, const float* finalValues, unsigned size, float time, float* result )
{
    //cerr<<"interpolate, t= "<<time<<endl;
    for( unsigned i=0; i<size; ++i )
        result[i] = (1-time)*initialValues[i] + time*finalValues[i];
}

// DOF values as a function of time
void Biped::update( float time, float dt )
{
    /// -------------- key positions ------------------
    enum { RIGHT_HIP, LEFT_HIP, NUM_DOFS };
    float keyValues[2][NUM_DOFS];
	
	// key position
	keyValues[0][RIGHT_HIP] = amplitude/2;
	keyValues[0][LEFT_HIP] = -amplitude/2;
	
	// key position
	keyValues[1][RIGHT_HIP] = -amplitude/2;
	keyValues[1][LEFT_HIP] = +amplitude/2;


    /// ------- finite state machine -------------------------
    enum { LeftStance, RightStance }; // behaviors
    static int current_behavior = LeftStance; // initial state = LeftStance
	int new_behavior;

    /// update the behavior (i.e. the state) based on time (and possibly other criteria)
	float t = time/period - floor(time/period); // to interval [0;1]
    if( current_behavior == LeftStance )
    {
        if( t>=0.5 )
		{
            new_behavior = RightStance;
		}
		else new_behavior = current_behavior;
	}
    else // current_behavior == RightStance
    {
		if( t<0.5 )
		{
            new_behavior = LeftStance;
		}
		else new_behavior = current_behavior;
	}
	current_behavior = new_behavior;

    /// -------- apply the behavior, i.e. compute the new joint angles -------
    float interpolated[NUM_DOFS];

    if( current_behavior == LeftStance ) // interpolate from first to second key position
    {
		this->interpolate( keyValues[0], keyValues[1], NUM_DOFS, t/0.5, interpolated );
    }
	else  // interpolate from second to first key position 
    {
		this->interpolate( keyValues[1], keyValues[0], NUM_DOFS, (t-0.5f)/0.5, interpolated  );
    }

    /// ------- apply the joint angles ---------
	right_hip->rotation.setValue( SbVec3f(1,0,0), interpolated[RIGHT_HIP] );
	left_hip->rotation.setValue( SbVec3f(1,0,0), interpolated[LEFT_HIP] );


    /// ------ displace the body -----------------------
    float step_length = sin(amplitude) * (lthigh+ltibia);
    float velocity = 2 * step_length / period; // average velocity
    float displacement = velocity * dt;
	
	bodyCenter += SbVec3f( 1,0,0 ) * displacement;
	position->translation = bodyCenter;

}
