#ifndef Biped_h
#define Biped_h

/**
A walking biped, with main frame centered in the middle of the body

@author Francois Faure
*/

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoMaterial.h>

class Biped
{
public:

    /// Constructeur using the dimensions of the character
    Biped(
        float hbody=3, float lbody=1.7, float pbody=0.8,  // body
        float rhead=0.5,                         // radius of the head
        float lthigh=2, float rthigh=0.45,     //  thigh
        float ltibia=1.9, float rtibia=0.22,     // tibia (under the knee)
        float lupper_arm=1.4, float rupper_arm=0.28,   // dimensions de la partie haute du upper_arm
        float llower_arm=1.3,                   // longueur de l'avant-upper_arm
        float period=2,                         // duree d'un cycle (deux pas)
        float amplitude = M_PI/3                 // angular amplitude of the hips
    );

    ~Biped();

    /// Update DOF values
    void update( float t, float dt);

	/// Reset biped position
	void reset();
	
	/// get the root node of the biped to insert it in a scene
	SoGroup* getRoot();




protected:

    SbVec3f bodyCenter; // center of the robo root

    float hbody, lbody, pbody;  // dimensions of the body
    float rhead;                   // radius of the head
    float lthigh, rthigh;        // dimensions ot the thigh
    float ltibia, rtibia;          // dimensions of the tibia
    float lupper_arm, rupper_arm;            // dimensions of the upper arm
    float llower_arm;             // length of the lower arm
    float period;                 // duration of a cycle (two steps)
    float amplitude;

    SoSeparator *root;
    SoTransform *position;   /// position of the character body wrt world
    SoTransform *right_hip;  
	SoTransform *left_hip;   
    SoTransform *right_knee ;
	SoTransform *left_knee ; 
    SoTransform *right_shoulder ;
    SoTransform *left_shoulder;
    SoTransform *right_elbow;
    SoTransform *left_elbow;

    // Apply a linear interpolation between 0 (initial values) and 1 (final values);
    void interpolate( const float* initialValues, const float* finalValues, unsigned size, float time, float* result );
};

#endif
