#ifdef HAVE_CONFIG_H
#include <config.h>
#endif




#include <stdlib.h>
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <Inventor/SoDB.h>
#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include "utils.h"   // lecture/ecriture de fichiers
#include "Biped.h"  // fonction de creation of the personnage
#include "argumentParser.h"  // fonction de creation of the personnage

SoSeparator* root;
SoDrawStyle* drawStyle;
SoTimerSensor* animClock;
float t=0;
float dt = 0.01;

// the character
Biped *biped;
bool running = false;




// scene update triggerd by the  SoTimerSensor* animClock
void
update(void *, SoSensor *)
{
    t += dt;
    biped->update(t,dt);
}

// process keyboard input
void
myKeyPressCB(void *, SoEventCallback *eventCB)
{
    const SoEvent *event = eventCB->getEvent();

    if (SoKeyboardEvent::isKeyPressEvent (event,SoKeyboardEvent::D))
        // mode d'affichage
    {
        static int mode = 0;  // valeur initiale:0
        if(mode==0)
        {
            drawStyle->style = SoDrawStyleElement::LINES;
            mode=1;
        }
        else if(mode==1)
        {
            drawStyle->style = SoDrawStyleElement::POINTS;
            mode=2;
        }
        else
        {
            drawStyle->style = SoDrawStyleElement::FILLED;
            mode=0;
        }
        eventCB->setHandled();
    }
    else if (SO_KEY_PRESS_EVENT(event, W))
        // sauvegarde de la scene
    {
        printf("Writing file scene.iv\n");
        writeFile(root,"scene.iv");
        eventCB->setHandled();
    }
    else if (SO_KEY_PRESS_EVENT(event, SPACE))
        // avancer d'un pas de temps
    {
        if(event->wasCtrlDown()){
            update( NULL, NULL );
            cerr<<"updated " << endl;
        }
        else {
        running = !running;
        if( running )
            animClock->schedule();
        else
            animClock->unschedule();
        }
        eventCB->setHandled();
    }
    else if (SO_KEY_PRESS_EVENT(event, R))
        // reinitialiser
    {
        t = 0;
        biped->reset();
        eventCB->setHandled();
    }
}


bool button1=false;
bool button2=false;
bool button3=false;

// process mouse button input
void
myMousePressCB(void *, SoEventCallback *eventCB)
{
    const SoEvent *event = eventCB->getEvent();

    if ( SoMouseButtonEvent::isButtonPressEvent(event,SoMouseButtonEvent:: BUTTON1) )
    {
        button1 = true;
        eventCB->setHandled();
    }
    if ( SoMouseButtonEvent::isButtonReleaseEvent(event,SoMouseButtonEvent:: BUTTON1) )
    {
        button1 = false;
    }
}


// process mouse motion input
void
myMouseMoveCB(void *, SoEventCallback */*eventCB*/)
{
//    const SoEvent *event = eventCB->getEvent();

//    if(button1)
//    {
//        SbVec2s location = event->getPosition();
//        std::cout<<"mouse at "<<location[0]<<", "<<location[1]<<endl;
//    }

}





int main(int argc, char **argv)
{
    // default character dimensions
    float hbody = 3;
    float lbody = 1.7;
    float pbody = 0.8;
    float rhead = 0.5;
    float lthigh = 2;
    float rthigh = 0.45;
    float ltibia = 1.9;
    float rtibia = 0.22;
    float lupper_arm = 1.4;
    float rupper_arm = 0.28;
    float llower_upper_arm =  1.3;
    float period = 2; // duration of the walk cycle
    float amplitude = M_PI/3;

    // read command line
    parse("This program simulates a parameterizable walking character. More detail on the command line is given below.\n \
          User interaction is the following:\n \
          Space: start/stop \n \
          Ctrl+Space: move one time step forward\n \
          Shift+Space: move one time step backwards\n \
          R: reset")
    .option(&dt,'t',"dt","time step")
    .option(&period,'p',"period","duration of one walking cycle")
    .option(&amplitude,'a',"amplitude","angular amplitude of the steps")
    .option(&hbody,0,"height_body","height of the body")
    .option(&lbody,0,"width_body","width of the body")
    .option(&pbody,0,"depth_body","depth of the body")
    .option(&rhead,0,"radius_head","radius of the head")
    .option(&lthigh,0,"length_thigh","length de la thigh")
    .option(&rthigh,0,"radius_thigh","radius de la thigh")
    .option(&ltibia,0,"length_tibia","length of the tibia")
    .option(&rtibia,0,"radius_tibia","radius of the tibia")
    .option(&lupper_arm,0,"length_upper_arm","length of the upper arm")
    .option(&rupper_arm,0,"radius_upper_arm","radius of the upper arm")
    .option(&llower_upper_arm,0,"length_lower_upper_arm","length of the lower arm")
    (argc,argv).printHelp();

    // Initialize the database and the graphics interface
    QWidget* myWindow = SoQt::init(argv[0]);
    if (myWindow == NULL)
        exit(1);

    // Build the scene ---------------------------------------------
    root = new SoSeparator;
    root->ref();

    // Visualize the world coordinate system
    SoSeparator* repere = readFile("coordinateSystemCyl.iv");
    root->addChild( repere );

    // draw style applied to the rest of the scene
    drawStyle = new SoDrawStyle;
    root->addChild( drawStyle );

    // the character
    biped = new Biped( hbody,lbody,pbody, rhead, lthigh,rthigh, ltibia,rtibia, lupper_arm,rupper_arm, llower_upper_arm, period, amplitude );
    root->addChild( biped->getRoot() );

    // the ground
    SoSeparator* sol = new SoSeparator;
    root->addChild(sol);

    float lsol = 5, Lsol=10;

    SoTexture2 *brique = new SoTexture2;
    sol->addChild( brique );
    brique->filename.setValue( "brick.rgb" );

    SoTexture2Transform* ttrans = new SoTexture2Transform;
    sol->addChild(ttrans);
    ttrans->scaleFactor.setValue(Lsol, lsol);

    SoTransform* tcube = new SoTransform;
    sol->addChild(tcube);
    tcube->translation.setValue(0,-0.5,0);

    SoCube* cube = new SoCube;
    sol->addChild(cube);
    cube->width.setValue(2*Lsol);
    cube->height.setValue(1);
    cube->depth.setValue(2*lsol);

    // ---end build scene ------------------------------------------

    // Event processing node (keyboard and mouse callbacks)
    SoEventCallback *myEventCB = new SoEventCallback;
    myEventCB->addEventCallback(
        SoKeyboardEvent::getClassTypeId(),
        myKeyPressCB, NULL);
    myEventCB->addEventCallback(
        SoMouseButtonEvent::getClassTypeId(),
        myMousePressCB, NULL);
    myEventCB->addEventCallback(
        SoLocation2Event::getClassTypeId(),
        myMouseMoveCB, NULL);
    root->addChild(myEventCB);

    // simulated time
    animClock = new SoTimerSensor( update, NULL ); // callback d'animation
    animClock->setInterval( dt );

    // the viewer
    SoQtExaminerViewer *myViewer = new SoQtExaminerViewer(myWindow);
    myViewer->setSceneGraph(root);
    myViewer->setTitle("Catwalk");
    myViewer->show();
    myViewer->viewAll();
    myViewer->setCameraType(SoOrthographicCamera::getClassTypeId() );
    myViewer->setViewing(false);
    //myViewer->setBackgroundColor(SbColor(1,1,1));

    // Run
    SoQt::show(myWindow);
    SoQt::mainLoop();

    return 0;
}



