//
// C++ Implementation: utils
//
// Description:
//
//
// Author: Francois Faure <Francois.Faure@imag.fr>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "utils.h"
#include <stdio.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoInput.h>
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

/** Read the file with given name and returns a pointer to the subgraph created.
Return Null if the graph could not be built.
*/
SoSeparator *
readFile(const char *filename)
{
   // Open the input file
   SoInput mySceneInput;
   if (!mySceneInput.openFile(filename)) {
      cerr<<"Cannot open file "<<filename<<endl;
      return NULL;
   }

   // Read the whole file into the database
   SoSeparator *myGraph = SoDB::readAll(&mySceneInput);
   if (myGraph == NULL) {
      cerr<<"Could not build graph from file "<<filename<<endl;
      cerr<<"Check if the format confroms to OpenInventor "<<endl;
      return NULL;
   }

   mySceneInput.closeFile();
   return myGraph;
}


/** Write the graph to a file.
\param root root of the graph to write
\param filename name of the written file
\param ascii if true write in ascii else write in binary format
*/
void writeFile( SoGroup* root, const char* filename, bool ascii )
{
        SoOutput out;
        if( !out.openFile(filename) ){
                cerr<<"Cannot open file "<<filename<<" for writing"<<endl;
        }
        SoWriteAction myWriteAction(&out);
        if( ascii )
            myWriteAction.getOutput()->setBinary( false );
        else
            myWriteAction.getOutput()->setBinary( true );
        myWriteAction.apply(root);
}



/// write a SbVec3f to an output flow
inline std::ostream& operator << (std::ostream& out, const SbVec3f& v )
{
        out << v[0] <<" "<< v[1] <<" "<< v[2];
        return out;
}

/// read a SbVec3f from an input flow
inline std::istream& operator >> (std::istream& in, SbVec3f& v )
{
        in >> v[0] >> v[1] >> v[2];
        return in;
}
